open Transition
open Letters
open Parser
open Tables
open Printer
open Tree

let index = ref 0

(* Rule: if STRING is on top, then the word is not closed *)
(* We close it as soon as we push a tag action *)

let glob = Stacklist.create ()

let set_string () = Stacklist.push STR (snd (Stacklist.top glob))

(* action1 is <color1,size1,optplain1,base1> </base1,optplain1,color1,size1>
   action2 is <color2,size2,optplain2,base2> </base2,optplain2,color2,size2>
   we want to transform it into

                action1 inter action2
               /                     \
      action1 minus action2      action2 minus action1
*)
   
let rec intersect_list l = function
  | [] -> (false,l,[],[])
  | tk::r ->
      try 
	let l' = remove tk l in
	let (_,l'',c',r') = intersect_list l' r in
	  (true,l'',tk::c',r')
      with Not_found ->
	let (ismod,l',c',r') = intersect_list l r in
	  (ismod,l',c',tk::r')

open Status.Act

let intersect act1 act2 =
  let ract1 = ref act1 in
  let inter = ref empty_action in
  let ract2 = ref act2 in
  let ismod = ref false in
  if is_size_defined act1 then
    (let sz1 = get_size act1 in
    if sz1 = get_size act2 then 
      (ismod:= true; inter := set_size !inter sz1;
      ract1 := unset_size !ract1; ract2 := unset_size !ract2));
  if is_color_defined act1 then
    (let c1 = get_color act1 in
    if c1 = get_color act2 then 
      (ismod:= true; inter := set_color !inter c1;
      ract1 := unset_color !ract1; ract2 := unset_color !ract2));
  if is_plain_set act1 && is_plain_set act2 then 
    (ismod:= true; inter := set_plain !inter;
    ract1 := unset_plain !ract1; ract2 := unset_plain !ract2);
  if not (is_plain_set !ract1) && not (is_plain_set !ract2) then 
    begin (* if a PL surrounds the action, no contraction below *)
      let (ismod',l1,l,l2) = intersect_list act1.abase act2.abase in
      if ismod' then
	(ract1 := set_base !ract1 l1; ract2 := set_base !ract2 l2;
	inter := set_base !inter l; ismod:= true)
    end;
  (!ismod, !ract1, !inter, !ract2)

let is_not_empty act = not (act = empty_action)

let rec push_tagged_node tklr sonsr brothers = (* tklr known not empty, sonsr never empty *)
  match Stacklist.decomp brothers with
    | Stacklist.Zero -> Stacklist.push (Node (tklr, sonsr)) brothers
    | _ ->
	(match Stacklist.top brothers with
	   | STR -> Stacklist.push (Node (tklr, sonsr)) brothers
	   | Node (tkll, sonsl) ->
	       let (nonempty,tkll',tklc,tklr') = intersect tkll tklr in
	       if nonempty then
		 (let midsons = Stacklist.create () in
		 push_node tkll' sonsl midsons;
		 push_node tklr' sonsr midsons; 
		 ignore (Stacklist.pop brothers);
		 Stacklist.push (Node (tklc, midsons)) brothers)
	       else
		 Stacklist.push (Node (tklr, sonsr)) brothers)

and push_node tkl sons brothers = (* tkl maybe empty, sons never empty *)
  if is_not_empty tkl then
    push_tagged_node tkl sons brothers
  else 
    Stacklist.iter (push_brother brothers) sons

and push_brother brothers = function
  | STR -> Stacklist.push STR brothers
  | Node (tkl, sons) -> push_tagged_node tkl sons brothers

let set st = function
  | Open tk ->
      let s = Stacklist.create () in
      Stacklist.push (st,s) glob
  | Closed tk ->
      let st,sons = Stacklist.pop glob in
      (match (Stacklist.decomp sons) with 
	| Stacklist.Zero -> ()
	| (Stacklist.One (Node (tkl, tl))) ->
	    push_node (contract st tkl tk) tl (snd (Stacklist.top glob))
	| Stacklist.Many | (Stacklist.One STR) -> 
	    push_node (make_action tk) sons (snd (Stacklist.top glob)))
  | Notag -> failwith "Should not occur"

let push_action (new_string_started,st,act) =
  if new_string_started then set_string ();
  set st act

let main () =
  set_initial_status ();
  Stacklist.push (Status.initial_status (), Stacklist.create ()) glob;
  parsing_state := ReadingWord;
  try
    while true do
      push_action (parse_action ())
    done
  with
    | End_of_file ->
	set_string ();
	print_document (snd (Stacklist.top glob))

let _ = if (not !Sys.interactive) then main ()

