open Transition
open Letters
open Tables
open Status

let look_ahead () = input_char stdin

let last_space_status = ref None

type parsing_state = ReadingWord | ReadingActionWithInitialStatus of status

let parsing_state = ref ReadingWord

(* On privilgie la vitesse  la concision du code ! (mouais) *)
let parse_tag () =
  let tag =
    match look_ahead () with
      | 'B' -> open_bold_tag BO
      | 'E' ->
	  (match look_ahead () with
	    | 'M' -> open_emph_tag EMO
	    | _ -> failwith "Syntax error: M expected")
      | 'I' -> open_italic_tag IO
      | 'P' ->
	  (match look_ahead () with
	    | 'L' -> open_plain_tag PLO
	    | _ -> failwith "Syntax error: L expected")
      | 'S' -> open_strong_tag SO
      | 'T' ->
	  (match look_ahead () with
	    | 'T' -> open_tt_tag TTO
	    | _ -> failwith "Syntax error: T expected")
      | 'U' -> open_underline_tag UO
      | '0' -> open_size_tag S0O
      | '1' -> open_size_tag S1O
      | '2' -> open_size_tag S2O
      | '3' -> open_size_tag S3O
      | '4' -> open_size_tag S4O
      | '5' -> open_size_tag S5O
      | '6' -> open_size_tag S6O
      | '7' -> open_size_tag S7O
      | '8' -> open_size_tag S8O
      | '9' -> open_size_tag S9O
      | 'r' -> open_color_tag CRO
      | 'g' -> open_color_tag CGO
      | 'b' -> open_color_tag CBO
      | 'c' -> open_color_tag CCO
      | 'm' -> open_color_tag CMO
      | 'y' -> open_color_tag CYO
      | 'k' -> open_color_tag CKO
      | 'w' -> open_color_tag CWO
      | '/' -> begin
	  match look_ahead () with
	    | 'B' -> close_tag BO
	    | 'E' ->
		(match look_ahead () with
		  | 'M' -> close_tag EMO
		  | _ -> failwith "Syntax error: M expected")
	    | 'I' -> close_tag IO
	    | 'P' ->
		(match look_ahead () with
		  | 'L' -> close_tag PLO
		  | _ -> failwith "Syntax error: L expected")
	    | 'S' -> close_tag SO
	    | 'T' ->
		(match look_ahead () with
		  | 'T' -> close_tag TTO
		  | _ -> failwith "Syntax error: T expected")
	    | 'U' -> close_tag UO
	    | '0' -> close_tag S0O
	    | '1' -> close_tag S1O
	    | '2' -> close_tag S2O
	    | '3' -> close_tag S3O
	    | '4' -> close_tag S4O
	    | '5' -> close_tag S5O
	    | '6' -> close_tag S6O
	    | '7' -> close_tag S7O
	    | '8' -> close_tag S8O
	    | '9' -> close_tag S9O
	    | 'r' -> close_tag CRO
	    | 'g' -> close_tag CGO
	    | 'b' -> close_tag CBO
	    | 'c' -> close_tag CCO
	    | 'm' -> close_tag CMO
	    | 'y' -> close_tag CYO
	    | 'k' -> close_tag CKO
	    | 'w' -> close_tag CWO
	    | _ -> failwith "Syntax error: tag expected"
	end
      | _ -> failwith "Syntax error: tag expected"
  in
  match look_ahead () with
    | '>' -> tag
    | _ -> failwith "Syntax error: '>' expected"

(* Main parsing function *)
let rec parse_action () =
  match look_ahead () with
    | '<' -> (* Should come first *)
	let prev_st = !current_status (* doit etre fait avant parse_tag *) in
	(match parse_tag () with
	  | Notag ->
	      parse_action ()
	  | c ->
	      if !parsing_state = ReadingWord then
		begin
		  parsing_state := ReadingActionWithInitialStatus prev_st;
		  (true,prev_st,c)
		end
	      else
		(false,prev_st,c))
    | ('\033'..'\127') as c ->
	begin
	  (match !parsing_state with
	    | ReadingActionWithInitialStatus s
		when s <> !current_status ->
		(* We close preceding word and start a new one *)
		(close_word (); parsing_state := ReadingWord)
	    | ReadingActionWithInitialStatus _ 
	      (* Action may be postponed; don't need to push preceding word *)
	      (* and we are still waiting for an action *)
	    | ReadingWord ->
	      (* We continue on the current word *)
		());
	  last_space_status := None;
	  push_letter c;
	  parse_action ()
	end
    | (' ' | '\013' | '\010' | '\009') as c ->
	let curs = current_space_status () in
	if
	  (not (is_current_tt_set ())) && !last_space_status = Some curs
	then
	  parse_action ()
	else
	  begin
	    (match !parsing_state with
	      | ReadingActionWithInitialStatus s when space_filter s <> curs ->
		  (*Current actions are significant; need to start a new word*)
		  (close_word (); parsing_state := ReadingWord)
	      | ReadingWord | ReadingActionWithInitialStatus _ -> ());
	    push_letter c;
	    last_space_status := Some curs;
	    parse_action ()
	  end
    | _ -> failwith "Syntax error: unexpected char"

