open Transition

type color
type size

val size_of_token : tag -> size
val color_of_token : tag -> color

type base_status
type space_status
type status

val initial_color : color
val initial_size : size
val initial_base : base_status
val initial_status : unit -> status
val copy_status : status -> status
val set_base : status -> base_status -> status
val get_base : status -> base_status
val is_tt_set : status -> bool
val is_bold_set : status -> bool
val is_strong_set : status -> bool
val is_italic_set : status -> bool
val is_plain_set : status -> bool
val get_underline : status -> int
val get_color : status -> color
val get_size :  status -> size
val set_tt : status -> status
val set_bold : status -> status
val set_strong : status -> status
val set_italic : status -> status
val unset_tt : status -> status
val unset_bold : status -> status
val unset_strong : status -> status
val unset_italic : status -> status
val toggle_em : status -> status
val incr_underline : status -> status
val decr_underline : status -> status
val set_plain : status -> status
val set_color : status -> color -> status
val set_size : status -> size -> status
val print_status : status -> unit
val space_filter : status -> space_status
val set_remove : status -> status
val is_remove_set : status -> bool

module Act : sig
  type action = {
    mutable aplain : bool;
    mutable abase : tag list;
    mutable asize : size;
    mutable acolor : color 
  }

  val empty_action : action      
  val is_plain_set : action -> bool
  val set_plain : action -> action
  val toggle_em : action -> action
  val add_base : tag -> action -> action
  val set_base : action -> tag list -> action
  val unset_plain : action -> action

  val set_size : action -> size -> action
  val set_color : action -> color -> action
  val unset_size : action -> action
  val unset_color : action -> action
  val get_size : action -> size
  val get_color : action -> color
  val is_size_defined : action -> bool
  val is_color_defined : action -> bool

  val print_close_action : action -> unit
  val print_open_action : action -> unit
end
