open Transition
open Status

let current_status = ref (initial_status ())

let is_current_tt_set () = is_tt_set !current_status
let is_current_bold_set () = is_bold_set !current_status
let is_current_strong_set () = is_strong_set !current_status
let is_current_italic_set () = is_italic_set !current_status

let is_current_plain_set () = is_plain_set !current_status
let current_underline () = get_underline !current_status
let current_color () = get_color !current_status
let current_size () = get_size !current_status

let set_current_tt () = current_status := set_tt !current_status
let set_current_bold () = current_status := set_bold !current_status
let set_current_strong () = current_status := set_strong !current_status
let set_current_italic () = current_status := set_italic !current_status

let toggle_current_em () = current_status := toggle_em !current_status
let incr_current_underline () = current_status :=incr_underline !current_status
let set_current_plain () = current_status := set_plain !current_status
let set_current_color c = current_status := set_color !current_status c
let set_current_size sz = current_status := set_size !current_status sz

let open_tag_stack = Stack.create ()
let push_open_tag c = Stack.push !current_status open_tag_stack
let push_removed_tag () = Stack.push (set_remove !current_status) open_tag_stack 
let pop_open_tag () = Stack.pop open_tag_stack

let current_space_status () = space_filter !current_status

let set_initial_status () = current_status := initial_status ()
let remember_status () = copy_status !current_status


(* We check for redundancies: if a tag set some values already set,
   then we skip the tag and push a mark to remove the correspondant
   closing tag *)

let open_plain_tag act =
  if is_current_plain_set () then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_plain (); Open act)

let open_bold_tag act =
  if is_current_bold_set () then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_bold (); Open act)

let open_tt_tag act =
  if is_current_tt_set () then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_tt (); Open act)

let open_emph_tag act =
  if is_current_strong_set () then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; toggle_current_em (); Open act)

let open_strong_tag act =
  if is_current_strong_set () then
    (push_removed_tag (); Notag)
  else
    (* Pas besoin de remettre em a 0: il n'influe pas sur les blancs *)
    (push_open_tag act; set_current_strong (); Open act)

let open_italic_tag act =
  if is_current_italic_set () then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_italic (); Open act)

let open_underline_tag act =
  if current_underline () = 3 then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; incr_current_underline (); Open act)

let open_color_tag act =
  if current_color () = color_of_token act then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_color (color_of_token act); Open act)

let open_size_tag act =
  if current_size () = size_of_token act then
    (push_removed_tag (); Notag)
  else
    (push_open_tag act; set_current_size (size_of_token act); Open act)

let close_tag act =
  let s = pop_open_tag () in
  if is_remove_set s then
    Notag
  else
    (current_status := s; (Closed act))


