open Transition
open Status

(* Rem: we assume the added action is not already here (except EM)
   because the parse ensures each token locally modifies the status *)

let contract st act = function 
  |EMO ->
     if Act.is_plain_set act then act
     else Act.toggle_em act 
  |(BO|IO|UO|TTO|SO) as tk ->
     if Act.is_plain_set act then act
     else Act.add_base tk act
  |PLO ->
     Act.set_plain act
  |(S0O|S1O|S2O|S3O|S4O|S5O|S6O|S7O|S8O|S9O) as tk ->
     if not (Act.is_size_defined act) then Act.set_size act (size_of_token tk)
     else
       if st = set_size st (Act.get_size act) then Act.unset_size act
       else act
  |(CRO|CGO|CBO|CCO|CMO|CYO|CKO|CWO) as tk ->
     if not (Act.is_color_defined act) then Act.set_color act (color_of_token tk)
     else
       if st = set_color st (Act.get_color act) then Act.unset_color act
       else act
  |_ -> failwith "Unexpected token"

let make_action = function
  |(EMO|BO|IO|UO|TTO|SO) as tk ->
     Act.add_base tk Act.empty_action
  |PLO ->
     Act.set_plain Act.empty_action
  |(S0O|S1O|S2O|S3O|S4O|S5O|S6O|S7O|S8O|S9O) as tk ->
     Act.set_size Act.empty_action (size_of_token tk)
  |(CRO|CGO|CBO|CCO|CMO|CYO|CKO|CWO) as tk ->
     Act.set_color Act.empty_action (color_of_token tk)
  |_ -> failwith "Not a token"

type tree =
  | STR
  | Node of Act.action * (tree Stacklist.t)
